package org.cebs.corep.xbrl.action;

import java.rmi.RemoteException;
import java.util.Hashtable;
import java.util.Iterator;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.struts.action.ActionError;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import org.cebs.corep.xbrl.exceptions.CorepBusinessException;
import org.cebs.corep.xbrl.exceptions.CorepResourceException;
import de.bundesbank.util.webgui.LogAction;
import org.cebs.corep.xbrl.util.CorepClientContext;
import de.bundesbank.util.log.ClientContext;

import de.bundesbank.util.webgui.struts.BasisAction;
import de.bundesbank.util.webgui.struts.BasisException;

/**
 * Euro-Evidenz Implementierung der Klasse <code>Action</code> fr das GUI Framework der LZB-BW
 *
 * @author    Thomas Brll
 * @author    Peter Waltenberger
 */
public abstract class CorepAction extends BasisAction implements LogAction
{
  /**
   * Diese Methode verarbeitet Fehlermeldungen, die in der GUI aufgetreten bzw.
   * an sie weitergereicht wurden.
   * <br />
   * 
   * @param  exception  BasisException
   * @param  mapping    ActionMapping
   * @param  form       ActionForm
   * @param  request    HttpServletRequest
   * @param  response   HttpServletResponse
   * @param  errors     ActionErrors
   * @return            ActionForward
   */
  public ActionForward handleBasisException(BasisException exception, ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response, ActionErrors errors)
  {
    String methodName = "handleBasisException";
    ActionForward result = null;
    Throwable detailException = exception.getDetail();

    // wird eine EuroResourceException geworfen, wird ein Admin-Fehler
    // an der Oberflche angezeigt.
    if (detailException instanceof CorepResourceException)
    {
      errors.add("resourceException", new ActionError("EuroFehler_1"));
      saveErrors(request, errors);
      log("BASIS_FRAMEWORK", getClass().getName(), methodName, detailException);
    }
    // im Falle einer Business-Exception wird die fachliche Fehlermeldung,
    // die bergeben wurde auf einer Fehlerseite dargestellt.
    else if (detailException instanceof CorepBusinessException)
    {
      // cast in eine EuroBusinessException
      CorepBusinessException businessException = (CorepBusinessException)detailException;
      // die Hashtable der EuroBusinessException wird nach Fehlermeldungen durchsucht
      Hashtable errorTable = businessException.getErrorTable();
      if (errorTable != null)
      {
        Iterator errorsIt = errorTable.keySet().iterator();
        while (errorsIt.hasNext())
        {
          String key = (String)errorsIt.next();
          // die gefundenen Fehlermeldungen werden als Action-Error-Objekte 
          // an die GUI weitergegeben.
          errors.add("corepBusinessException", ExceptionTransformer.getActionError(key, (Object[])errorTable.get(key)));
        }
      }
      else
      {
        errors.add("corepBusinessException", ExceptionTransformer.getActionError(businessException.getMessage(), businessException.getParams()));
      }
      // stellt die Fehlermeldungen in das Request-Objekt ein
      saveErrors(request, errors);

      // im Falle, dass eine Seite in der struts-config angegeben wurde, in der
      // der Fehler angezeigt werden soll, wird an diese weitergeleitet.
      if (mapping.getInput() != null && mapping.getInput().length() > 0)
      {
        // Erzeugen eines ActionForward-Objektes, das zu der JSP
        // zeigt, die im Action-Parameter "input" spezifiziert ist
        result = new ActionForward(mapping.getInput());
        result.setName("input");
      }
    }
    // wird eine RemoteException geworfen, wird ein Admin-Fehler angezeigt,
    // und der Fehler als remotExecption weitergereicht geloggt.
    else if (detailException instanceof RemoteException)
    {
      errors.add("remoteException", new ActionError("EuroFehler_1"));
      saveErrors(request, errors);
      log("BASIS_FRAMEWORK_ERROR", getClass().getName(), methodName, detailException);
    }
    // bei allen anderen Exceptions, wird ebenfalls ein Admin-Fehler angezeigt,
    // und der Fehler als generalException weitergereicht.
    else
    {
      errors.add("generalException", new ActionError("EuroFehler_1"));
      saveErrors(request, errors);
      log("BASIS_FRAMEWORK_ERROR", getClass().getName(), methodName, detailException);
    }

    // wenn kein Input angegeben wurde, Anzeige der Error-Page
    if (result == null)
    {
      result = mapping.findForward("failure");
    }

    return result;
  }

  /**
   * ClientContext erzeugen und an SuperKlasse zurckgeben
   * <br />
   * @param  mapping               ActionMapping
   * @param  form                  ActionForm
   * @param  request               HttpServletRequest
   * @param  response              HttpServletResponse
   * @return                       de.bundesbank.util.log.ClientContext
   * @exception  ServletException  
   */
  public ClientContext initClientContext(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response) throws ServletException
  {
    CorepClientContext clientContext;

    // der EuroClientContext wird aus dem Request geholt
    clientContext = (CorepClientContext)request.getSession().getAttribute("ClientContext");

    // falls der EuroClientContext noch null ist, wird der User und die Session-Id gesetzt
    if (clientContext == null)
    {
      clientContext = new CorepClientContext(); // Initialisierung ClientContext
      clientContext.setUserID(request.getRemoteUser());
      clientContext.setSessionID(request.getSession().getId());
      request.getSession().setAttribute("ClientContext", clientContext);
    }
    // eine Kopie des ClientContexts wird bergeben
    ClientContext ctxCopy = (ClientContext)clientContext.clone();
    return ctxCopy;
  }

  /**
   * Diese Methode muss von allen Unterklassen implementiert werden
   * und enthaelt die eigentliche Business Logik der jeweiligen <code>Action</code>,
   * die ausgefuehrt werden soll, falls die Validierung erfolgreich war.
   * <br />
   *
   * @param  mapping                     ActionMapping
   * @param  form                        ActionForm
   * @param  request                     HttpServletRequest
   * @param  response                    HttpServletResponse
   * @exception  EuroBusinessException  
   * @exception  RemoteException         
   */
  public abstract void performAction(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response) throws CorepBusinessException, RemoteException;

  /**
   * Implementierung der BasisAction fr die Euro-Evidenz
   * Sie enthaelt die Business Logik der jeweiligen <code>Action</code>,
   * die ausgefuehrt werden soll, falls die Validierung erfolgreich war.
   * Hier ist es mglich den Forward abhngig vom jeweiligen Ergebnis
   * zu setzen.
   * <br />
   *
   * @param  mapping             ActionMapping
   * @param  form                ActionForm
   * @param  request             HttpServletRequest
   * @param  response            HttpServletResponse
   * @return                     ActionForward
   * @exception  BasisException  
   */
  public ActionForward performExecute(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response) throws BasisException
  {
    try
    {
      performAction(mapping, form, request, response);
      return mapping.findForward("success");
    }
    catch (Exception e)
    {
      log("BASIS_FRAMEWORK", getClass().getName(), "performExecute", e);
      throw new BasisException(e);
    }

  }

  /**
   * Diese Methode muss von allen Unterklassen implementiert werden
   * und enthaelt die Logik der jeweiligen <code>Action</code>, die
   * ausgefuehrt werden soll, falls die Validierung fehlgeschlagen ist.
   * <br />
   *
   * @param  mapping   ActionMapping
   * @param  form      ActionForm
   * @param  request   HttpServletRequest
   * @param  response  HttpServletResponse
   */
  public void performReset(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response)
  {
  }

  /**
   * Leere Implementierung
   * <br />
   *
   * @param  mapping  ActionMapping
   * @param  form     ActionForm
   * @param  request  HttpServletRequest
   */
  public void performResetForm(ActionMapping mapping, ActionForm form, HttpServletRequest request)
  {
  }

  /**
   * Diese Methode ist als Template fuer die zu implementierenden <code>Action</code>-Klassen gedacht.
   * Sie ist in diesen bei Bedarf individuell zu implementieren.
   * <br />
   * Da diese Methode als Template fungiert, ist keine Funktionalitaet implementiert. Ihr
   * Aufruf reicht einfach das uebergebene <code>ActionErrors</code>-Objekt durch.
   * <br />
   *
   * @param  errors                      ActionErrors
   * @param  form                        ActionForm
   * @return                             ActionErrors
   * @exception  EuroBusinessException  
   * @exception  RemoteException         
   */
  public ActionErrors validate(ActionErrors errors, ActionForm form) throws CorepBusinessException, RemoteException
  {
    return errors;
  }

  /**
   * Diese Methode ist als Template fuer die zu implementierenden <code>Action</code>-Klassen gedacht.
   * Sie ist in diesen bei Bedarf individuell zu implementieren.
   * <br />
   * Da diese Methode als Template fungiert, ist keine Funktionalitaet implementiert. Ihr
   * Aufruf reicht einfach das uebergebene <code>ActionErrors</code>-Objekt durch.
   * <br />
   *
   * @param  mapping                     ActionMapping
   * @param  form                        ActionForm
   * @param  request                     HttpServletRequest
   * @param  response                    HttpServletResponse
   * @param  errors                      ActionErrors
   * @return                             ActionErrors
   * @exception  EuroBusinessException  
   * @exception  RemoteException         
   */
  public ActionErrors validate(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response, ActionErrors errors) throws CorepBusinessException, RemoteException
  {

    return validate(errors, form);
  }

  /**
   * Diese Methode validiert die Form und gibt ein Objekt vom Typ
   * ActionErrors zurck.
   * <br />
   * @param  mapping             ActionMapping
   * @param  form                ActionForm
   * @param  request             HttpServletRequest
   * @param  response            HttpServletResponse
   * @param  errors              ActionErrors
   * @return                     ActionErrors
   * @exception  BasisException  
   */
  public ActionErrors validateForm(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response, ActionErrors errors) throws BasisException
  {
    ActionErrors result = errors;
    try
    {
      result = validate(mapping, form, request, response, errors);
    }
    catch (Exception e)
    {
      throw new BasisException(e);
    }
    return result;
  }

  /**
   * Return den ClientContext, den die Superklasse zur Verfgung stellt
   *
   *
   * @return    ClientContext
   */
  public ClientContext getClientContext()
  {
    return (ClientContext)getIClientContext();
  }

}
