package org.cebs.corep.xbrl.instanceGenerator;

import org.apache.commons.lang.math.NumberUtils;
import org.apache.commons.lang.StringUtils;
import org.cebs.corep.excelToXbrl.constants.XbrlConstantsInterface;
import org.jdom.Element;
import org.jdom.Namespace;

import com.ibm.ws.ejbpersistence.dataaccess.DoubleFieldExtractor;

/**
 * This class builds an xml jdom tree out of a list of HSSFRow objects.
 */
public class ElementCreator implements XbrlConstantsInterface
{
  private static final String CLASSNAME = "ElementCreator";
  /**
   * Builds an element, adds an context attribute to this element and 
   * appends it to the parent element without any text content.
   * 
   * @param parentElement
   * @param content
   * @param tagName
   * @param ns
   * @param context
   */
  protected static Element createElement(Element parentElement, String content, String tagName, Namespace ns, String context)
  {
    Element element = null;
    if (StringUtils.isNotBlank(content))
    {
      // builds new element
      element = new Element(tagName, ns);
      // adds a context-Attribut h
      element.setAttribute(CONTEXTREF, context);
      // appends the element to the parent
      parentElement.addContent(element);
    }
    return element;
  }

  /**
   * This method creates a non numeric xbrl element with text content.
   *  
   * @param parentElement		Element
   * @param content 			String
   * @param tagName 			String
   * @param ns					Namespace
   * @param context				String
   */
  protected static void createUniversalElement(Element parentElement, String content, String tagName, Namespace ns, String context)
  {
    // creates a non numeric xbrl element
    Element element = createElement(parentElement, content, tagName, ns, context);
    if (element != null)
    {
      // adds content to the element
      element.addContent(content);
    }
  }

  /**
   * This method creates a monetary xbrl element with "0" decimals and the unit "EUR".
   *  
   * @param parentElement		Element
   * @param content 			String
   * @param tagName 			String
   * @param ns					Namespace
   * @param context				String
   */
  protected static void createMonetaryElement(Element parentElement, String content, String tagName, Namespace ns, String context)
  {
    // creates a non numeric xbrl element
    Element element = createElement(parentElement, content, tagName, ns, context);

    if (element != null)
    {
      // adds specific monetary attributes
      element.setAttribute(DECIMALS, "0");
      element.setAttribute(UNITREF, ATTRIBUTE_EUR_UNIT_CONTENT);
      // adds content to the element
      element.addContent(content);
    }
  }

  /**
   * This method creates a xbrl element for an element content of type double.
   *  
   * @param parentElement		Element
   * @param content 			String
   * @param tagName 			String
   * @param ns					Namespace
   * @param context				String
   */
  protected static void createDoubleElement(Element parentElement, String content, String tagName, Namespace ns, String context)
  {
    // creates a non numeric xbrl element
    Element element = createElement(parentElement, content, tagName, ns, context);

    if (element != null)
    {
      // adds specific attributes for doubleItemType elements
      element.setAttribute(DECIMALS, "2");
      element.setAttribute(UNITREF, ATTRIBUTE_PURE_UNIT_CONTENT);
      // adds content to the element
      element.addContent(content);
    }
  }

  /**
   * This method creates a xbrl element for an element content of type integer.
   *  
   * @param parentElement		Element
   * @param content 			String
   * @param tagName 			String
   * @param ns					Namespace
   * @param context				String
   */
  protected static void createIntegerElement(Element parentElement, String content, String tagName, Namespace ns, String context)
  {
    // creates a non numeric xbrl element
    Element element = createElement(parentElement, content, tagName, ns, context);

    if (element != null)
    {
      // adds specific attributes for integerItemType elements
      element.setAttribute(DECIMALS, "0");
      element.setAttribute(UNITREF, ATTRIBUTE_PURE_UNIT_CONTENT);
      // adds content to the element
      try
      {
        element.addContent(new Long(Math.round(new Double(content).doubleValue())).toString());
      }
      catch (NumberFormatException ne)
      {
        if (tagName.equalsIgnoreCase("Day"))
          element.addContent("11");
      }
    }
  }

  /**
   * This method creates a xbrl element for an element with percentage content.
   *  
   * @param parentElement		Element
   * @param content 			String
   * @param tagName 			String
   * @param ns					Namespace
   * @param context				String
   */
  protected static void createPercentageElement(Element parentElement, String content, String tagName, Namespace ns, String context)
  {
    // creates a non numeric xbrl element
    Element element = createElement(parentElement, content, tagName, ns, context);
    if (element != null)
    {
      // adds specific attributes
      element.setAttribute(DECIMALS, "2");
      element.setAttribute(UNITREF, ATTRIBUTE_PURE_UNIT_CONTENT);

      // transforms the character string content into a double multiplies
      // it with 100 and rounds the result to 2 decimals
      if (StringUtils.contains(content, ','))
      {
        content = StringUtils.replace(content, ",", ".");
      }
        
      double doubleValue = NumberUtils.createDouble(content).doubleValue();
      doubleValue = Math.round(doubleValue * 100.) / 100.;
      // adds content to the element
      element.addContent((new Double(doubleValue)).toString());
    }
  }

  /**
   * This method creates a xbrl element for an element content of type boolean.
   *  
   * @param parentElement		Element
   * @param content 	String
   * @param tagName 	String
   * @param ns			Namespace
   * @param context		String
   */
  protected static void createBooleanElement(Element parentElement, String content, String tagName, Namespace ns, String context)
  {
    //	creates a non numeric xbrl element
    Element element = createElement(parentElement, content, tagName, ns, context);

    if (element != null)
    {
      // is the content "yes" or "y" the value is set to "true", for "no" or "n" the
      // value is set to "false"
      if (StringUtils.contains(content, 'y') || StringUtils.contains(content, 'Y'))
        content = "true";
      else
        content = "false";
      // adds content to the element
      element.addContent(content);
    }

  }

  /**
   * This method creates a context element for a tuple based template.
   * @param contextContent
   * @param referenceNumber
   * @param instantDate
   * @return
   */
  public static Element createGeneralContextElement(String contextContent, String referenceNumber, String instantDate)
  {
    Namespace nsContext = Namespace.getNamespace(XBRLI_NS, XBRLI_NS_URL);

    // creates context element
    Element context = new Element(CONTEXT, nsContext);
    context.setAttribute(ATTRIBUTE_ID, contextContent);

    Element entity = new Element(ENTITY, nsContext);
    Element identifier = new Element(IDENTIFIER, nsContext);
    identifier.setAttribute(ATTRIBUTE_SCHEME, ATTRIBUTE_SCHEME_CONTENT);
    identifier.addContent(referenceNumber);
    entity.addContent(identifier);

    Element period = new Element(PERIOD, nsContext);
    Element instant = new Element(INSTANT, nsContext);
    instant.addContent(instantDate);
    period.addContent(instant);
    context.addContent(entity);
    context.addContent(period);

    // returns context element
    return context;
  }

  /**
   * Creates a euro unit element.
   *
   * @return Element
   */
  public static Element createUnitEurElement()
  {
    Namespace nsContext = Namespace.getNamespace(XBRLI_NS, XBRLI_NS_URL);

    // creates unit element
    Element unit = new Element(UNIT, nsContext);
    unit.setAttribute(ATTRIBUTE_ID, ATTRIBUTE_EUR_UNIT_CONTENT);
    Element measure = new Element(MEASURE, nsContext);
    measure.addContent(MEASURE_EUR_CONTENT);
    unit.addContent(measure);

    // returns unit element
    return unit;
  }

  /**
   * Create pure unit element.
   * 
   * @return Element
   */
  public static Element createUnitPureElement()
  {
    Namespace nsContext = Namespace.getNamespace(XBRLI_NS, XBRLI_NS_URL);

    // creates unit element
    Element unit = new Element(UNIT, nsContext);
    unit.setAttribute(ATTRIBUTE_ID, ATTRIBUTE_PURE_UNIT_CONTENT);
    Element measure = new Element(MEASURE, nsContext);
    measure.addContent(MEASURE_PURE_CONTENT);
    unit.addContent(measure);

    // returns unit element
    return unit;
  }

  /**
   * This method creates an element for the risk weight content of some templates.
   * 
   * @return Element
   */
  public static Element createRiskWeightElement(Element parentElement, String content, String tagName, Namespace ns, String context)
  {
    // creates a general element  
    Element element = createElement(parentElement, content, tagName, ns, context);

    if (element != null)
    {
      // transforms the contetn string in a double value and multiplies it
      // with 100; the value is rounded to 2 decimals
      try
      {
        double doubleValue = (new Double(content)).doubleValue();
        doubleValue = Math.round(doubleValue * 100. * 100.) / 100.;
        // adds content
        element.addContent((new Double(doubleValue)).toString() + "%");
      }
      catch (NumberFormatException e)
      {
        element.addContent(content);
      }
    }
    return element;
  }

}
