package org.cebs.corep.xbrl.instanceGenerator;

import java.util.ArrayList;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Properties;

import org.apache.commons.collections.IterableMap;
import org.apache.commons.collections.MapIterator;
import org.apache.commons.collections.map.HashedMap;
import org.cebs.corep.excelToXbrl.constants.XbrlConstantsInterface;
import org.cebs.corep.excelToXbrl.mapper.RowsToXbrlMapper;
import org.cebs.corep.xbrl.log.CorepLogger;
import org.cebs.corep.xbrl.log.LogAction;
import org.cebs.corep.xbrl.util.CorepClientContext;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.Namespace;

/**
 * This class offers functionality for the creation of xbrl instance documents.
 * The data of the coreptest file is read table for table and transformed 
 * into XML/XBRL.
 */
public class InstanceGenerator implements XbrlConstantsInterface
{
  private static final String CLASSNAME = "InstanceGenerator";

  /**
   * This method generates on basis of a corep test case a XBRL instance document.
   * @param workbookContent	HashMap 
   * @param cc				CorepClientContext
   * @return Document 		
   */
  public static Document generateXbrlInstance(HashMap workbookContent, CorepClientContext cc)
  {
    CorepLogger.log(LogAction.METHOD_BEGIN, CLASSNAME, "generateXbrlInstance", cc);

    String referenceNumber = RowsToXbrlMapper.getReferenceNumber(workbookContent, cc);
    String instantDate = RowsToXbrlMapper.getInstantDate(workbookContent, cc);

    // create the root element of the instance document
    Element xbrlRoot = new Element(XBRL, Namespace.getNamespace(XBRLI_NS, XBRLI_NS_URL));

    // create SchemaRef element
    ArrayList containedTemplates = createSchemaRefElements(xbrlRoot, workbookContent);

    // add the namespace declarations for all taxonomies that are needed for the test case
    setAdditionalNamespaceDeclaration(xbrlRoot, containedTemplates);

    // add the schemaLocation attribute to the xbrl root element
    addSchemaLocationNamespaceDeclaration(xbrlRoot, containedTemplates);

    // iterate over the content of the Excel workbook
     readExcelWorkbookContent(xbrlRoot, workbookContent, referenceNumber, instantDate, cc);

    // add unit element for EUR and PURE
    xbrlRoot.addContent(RowsToXbrlMapper.createUnitEurElement());
    xbrlRoot.addContent(RowsToXbrlMapper.createUnitPureElement());

    CorepLogger.log(LogAction.METHOD_END, CLASSNAME, "generateXbrlInstance", cc);

    // returns the xml document
    return new Document(xbrlRoot);
  }

  /**
   * This method iterates over the pages of the Excel workbook and creates the xbrl document.
   * @param xbrlRoot
   * @param workbookContent
   * @param referenceNumber
   * @param instantDate
   * @param cc
   */
  private static void readExcelWorkbookContent(Element xbrlRoot, HashMap workbookContent, String referenceNumber, String instantDate, CorepClientContext cc)
  {
     IterableMap map = new HashedMap(workbookContent);
      MapIterator it = map.mapIterator();
      int sheetsCounter = 1;
      while (it.hasNext())
      {
        // sheet name
        String tableName = it.next().toString();
        // sheet content
        Object tableContent = it.getValue();
    
        // create the xml representation of each test case table
        if (!tableName.equalsIgnoreCase(OVERVIEW_PAGE))
        {
          RowsToXbrlMapper.createXbrlTemplate(tableName, (ArrayList)tableContent, xbrlRoot, referenceNumber, instantDate, sheetsCounter, cc);
          sheetsCounter++;
        }
      }
  }

  /**
   * This method builds a properties object that combines the table 
   * name with short name of the template taxonomy
   * @return
   */
  private static Properties getTablenameTemplateTaxonomyProperties()
  {
    //	Properties object to map the table names to the according template taxonomies
    Properties prop = new Properties();
    prop.put(T_MKR_IM, T_MI_NS);
    prop.put(T_MKR_DI, T_MD_NS);
    prop.put(T_OPR, T_OP_NS);
    prop.put(T_OPR_DETAILS, T_OD_NS);
		prop.put(T_OPR_LOSS_DETAILS, T_OL_NS);
    prop.put(T_MKR_SA_TDI, T_MT_NS);
    prop.put(T_MKR_SA_EQU, T_ME_NS);
    prop.put(T_MKR_SA_COM, T_MC_NS);
    prop.put(T_MKR_SA_FX, T_MF_NS);

    return prop;
  }

  /**
   * This method builds a HashMap that contains the template taxonomy name as key
   * and its correspondend taxonomies as value.
   * @return
   */
  private static HashMap getCorrespondingTaxonomies()
  {
    //	Properties object to map the table names to the according template taxonomies
    HashMap map = new HashMap();
    map.put(T_MI_NS, new String[] { P_MI_NS, D_PM_NS });
    map.put(T_MD_NS, new String[] { P_MD_NS });
    map.put(T_OP_NS, new String[] { P_OP_NS, D_BA_NS, D_BL_NS });
    map.put(T_OD_NS, new String[] { P_OD_NS, D_BL_NS, D_EV_NS, D_TA_NS });
		map.put(T_OL_NS, new String[] { P_OL_NS });
    map.put(T_MT_NS, new String[] { P_MT_NS, D_RI_NS });
    map.put(T_ME_NS, new String[] { P_ME_NS, D_RI_NS });
    map.put(T_MC_NS, new String[] { P_MC_NS, D_PC_NS, D_MZ_NS });
    map.put(T_MF_NS, new String[] { P_MF_NS, D_RC_NS });

    return map;
  }

  /**
   * This method creates the XBRL schemaRef elements for each template taxonomy that
   * is needed for the test case.
   * @param xbrlRoot
   * @param workbookContent
   */
  private static ArrayList createSchemaRefElements(Element xbrlRoot, HashMap workbookContent)
  {
    // list for the taxonomies that are need for the test case
    ArrayList containedTemplates = new ArrayList();

    // create a properties object for the combination table name to taxonomy name
    Properties prop = getTablenameTemplateTaxonomyProperties();

    // create a hashMap for the template taxonomy and its correspondend measure/dimension(s) taxonomies
    HashMap taxonomyMap = getCorrespondingTaxonomies();

    // iterate over HashMap
    IterableMap map = new HashedMap(workbookContent);
    MapIterator it = map.mapIterator();
    while (it.hasNext())
    {
      String tableName = it.next().toString();

      // find table name in key objects of properties object
      for (Enumeration enum = prop.propertyNames(); enum.hasMoreElements();)
      {
        String template = (String)enum.nextElement();
        if (tableName.endsWith(template) && !containedTemplates.contains(prop.getProperty(template)))
        {
          // create a schemaRef element for the table
          Element schemaRef = new Element(SCHEMAREF, XBRLL_NS, LINKBASE_NS_URL);
          schemaRef.setAttribute(ATTRIBUTE_TYPE, ATTRIBUTE_CONTENT_TYPE, Namespace.getNamespace(XLINK_NS, XLINK_NS_URL));
          schemaRef.setAttribute(ATTRIBUTE_ARCROLE, ATTRIBUTE_CONTENT_ARCROLE, Namespace.getNamespace(XLINK_NS, XLINK_NS_URL));
          schemaRef.setAttribute(ATTRIBUTE_HREF, prop.getProperty(template) + NS_DATE + XSD, Namespace.getNamespace(XLINK_NS, XLINK_NS_URL));
          xbrlRoot.addContent(schemaRef);

          // add the template taxonomy name
          containedTemplates.add(prop.getProperty(template));

          // add the correspondend taxonomies of the template taxonomy to the list
          String[] correspondendTaxonomies = (String[])taxonomyMap.get(prop.getProperty(template));
          for (int i = 0; i < correspondendTaxonomies.length; i++)
          {
            containedTemplates.add(correspondendTaxonomies[i]);
          }
        }
      }
    }

    // return the list with all involved taxonomies
    return containedTemplates;
  }

  /**
   * This method creates the schemaLocation attribute for the XBRL root element so
   * that the instance document can be validated against xml.
   * @param xbrlRoot
   * @param taxonomies  ArrayList with the short names of the taxonomies
   */
  private static void addSchemaLocationNamespaceDeclaration(Element xbrlRoot, ArrayList taxonomies)
  {
    // declarations for the general taxonomies
    StringBuffer buffer = new StringBuffer();
    buffer.append(XBRLI_NS_URL + " xbrl-instance-2003-12-31.xsd ");
    buffer.append(" " + LINKBASE_NS_URL + " xbrl-linkbase-2003-12-31.xsd");
    buffer.append(" " + XLINK_NS_URL + " xlink-2003-12-31.xsd");
    buffer.append(" " + XBRLDI_NS_URL + " xbrldi-2005.xsd");
    buffer.append(" " + DT_NS_URL + " dt-2005-06-30.xsd");

    String namespaceURL = NS_URL;

    // declaration for all involved taxonomies
    Iterator iterator = taxonomies.iterator();
    while (iterator.hasNext())
    {
      String templateName = (String)iterator.next();
      buffer.append(" " + namespaceURL + templateName + NS_DATE + " " + templateName + NS_DATE + XSD);
    }

    // add to the schemaLocation attribute
    Namespace xsi_ns = Namespace.getNamespace(XSI_NS, XSI_NS_URL);
    xbrlRoot.setAttribute("schemaLocation", buffer.toString(), xsi_ns);

  }

  /**
   * This method adds the namespaces of all template, measure and dimension taxonomies to
   * the root element of the instance document.
   * @param xbrlRoot
   * @param taxonomies
   */
  private static void setAdditionalNamespaceDeclaration(Element xbrlRoot, ArrayList taxonomies)
  {
    // define the general namespace declarations
    xbrlRoot.addNamespaceDeclaration(Namespace.getNamespace(XBRLL_NS, LINKBASE_NS_URL));
    xbrlRoot.addNamespaceDeclaration(Namespace.getNamespace(XLINK_NS, XLINK_NS_URL));
    xbrlRoot.addNamespaceDeclaration(Namespace.getNamespace(ISO4217_NS, ISO4217_NS_URL));
    xbrlRoot.addNamespaceDeclaration(Namespace.getNamespace(DT_NS, DT_NS_URL));
    xbrlRoot.addNamespaceDeclaration(Namespace.getNamespace(XBRLDI_NS, XBRLDI_NS_URL));
    xbrlRoot.addNamespaceDeclaration(Namespace.getNamespace(XSI_NS, XSI_NS_URL));

    String namespaceURL = NS_URL;

    //	add the taxonomie declarations for all involved templates
    Iterator iter = taxonomies.iterator();
    while (iter.hasNext())
    {
      String taxonomyName = (String)iter.next();
      xbrlRoot.addNamespaceDeclaration(Namespace.getNamespace(taxonomyName, namespaceURL + taxonomyName + NS_DATE));
    }
  }
}
